<?php

namespace Drupal\entity_ui\EntityHandler;

use Symfony\Component\Routing\RouteCollection;

/**
 * Proxy handler for entity types with a field UI, but no bundle entities.
 *
 * This handles the two cases that can't distinguish between during entity type
 * build, as we either need to inspect the field UI base route or get bundle
 * info, neither of which can be done during entity type build because it would
 * cause circularity.
 */
class FieldUIWithoutBundleEntityProxy extends EntityUIAdminBase {

  /**
   * The real handler that this wraps.
   *
   * @var \Drupal\entity_ui\EntityHandler\EntityUIAdminInterface
   */
  protected $realHandler;

  /**
   * Override this so we instantiate the real handler.
   *
   * Routes are built before the link entities, so this is the first call to
   * this handler.
   */
  public function getRoutes(RouteCollection $route_collection) {
    $field_ui_base_route_name = $this->entityType->get('field_ui_base_route');

    // Figure out the real handler to use and instantiate it.
    $field_ui_base_route = $route_collection->get($field_ui_base_route_name);
    $field_ui_base_route_path = $field_ui_base_route->getPath();

    if (!empty($field_ui_base_route_path) && substr($field_ui_base_route_path, -(strlen('/{bundle}'))) == '/{bundle}') {
      // The entity type doesn't have a bundle entity type, but has multiple
      // bundles. These might be derived from plugins (using the Entity API
      // contrib module's functionality), or simnply hardcoded in
      // hook_entity_bundle_info(). We detect this by the presence of a 'bundle'
      // parameter at the end of the field UI route path, which Field UI module
      // expects when the bundles are not config entities.
      $handler_class = \Drupal\entity_ui\EntityHandler\PlainBundlesEntityUIAdmin::class;
    }
    else {
      // The entity type has only a single bundle.
      $handler_class = \Drupal\entity_ui\EntityHandler\BasicFieldUI::class;
    }

    $this->realHandler = $this->entityTypeManager->createHandlerInstance($handler_class, $this->entityType);

    return $this->realHandler->getRoutes($route_collection);
  }

  /**
   * This needs to be implemented here because it's abstract in the base class.
   */
  protected function getCollectionRoutePath(RouteCollection $route_collection) {
    return $this->realHandler->getCollectionRoutePath($route_collection);
  }

  /**
   * Call through to the real handler.
   */
  function __call($name, $arguments) {
    return $this->realHandler->{$name}(...$arguments);
  }

}
